@@ -3,6 +3,7 @@ use crate::Status;
 use anyhow::{anyhow, Context, Result};
 use chrono::{DateTime, Utc};
 use colored::*;
+use log::info;
 use serde::{Deserialize, Serialize};
 use std::fmt::{Debug, Display, Formatter};
 use std::path::{Path, PathBuf};
@@ -27,6 +28,7 @@ pub struct Command {
     os: String,
 }
 
+// associated functions
 impl Command {
     pub fn new(name: &str) -> Self {
         let mut d = Command::default();
@@ -46,6 +48,206 @@ impl Command {
         d
     }
 
+    pub fn collect(search: &str, limit: usize) -> Result<Vec<Command>> {
+        Storage::search(search, limit)
+            .with_context(|| format!("Command::collect('{}',{})", search, limit))
+    }
+
+    fn which(name: &str) -> Result<PathBuf> {
+        let extensions = vec![".exe", ".bat", ".cmd", ""];
+        for ext in extensions.iter() {
+            let exe_name = format!("{}{}", name, ext);
+            match Command::which_exact(&exe_name) {
+                Some(path) => {
+                    return Ok(path);
+                }
+                None => {}
+            };
+        }
+        Err(anyhow!("unrecognized command '{}'", name))
+    }
+
+    fn which_exact(name: &str) -> Option<PathBuf> {
+        std::env::var_os("PATH").and_then(|paths| {
+            std::env::split_paths(&paths)
+                .filter_map(|dir| {
+                    let full_path = dir.join(&name);
+                    if full_path.is_file() {
+                        Some(full_path)
+                    } else {
+                        None
+                    }
+                })
+                .next()
+        })
+    }
+
+    /*
+    pub fn format(&self, fmt: &str) -> String {
+        let duration = self.duration_string();
+        let cur_dir = match &self.working_dir {
+            Some(dir) => format!("{}", dir.display()),
+            None => "".to_string(),
+        };
+        if fmt == "%F" {
+            if cur_dir.len() > 0 {
+                format!(
+                    "{} {} {} ({},{})",
+                    &self.status(),
+                    &self.name().yellow().bold(),
+                    &self.args.join(" "),
+                    cur_dir.white().bold(),
+                    duration.white().bold(),
+                )
+            } else {
+                format!(
+                    "{} {} {} ({})",
+                    &self.status(),
+                    &self.name().yellow().bold(),
+                    &self.args.join(" "),
+                    duration.white().bold(),
+                )
+            }
+        } else {
+            format!("{}", self)
+        }
+    }*/
+}
+
+// immutable methods
+impl Command {
+    pub fn name(&self) -> &str {
+        &self.name
+    }
+
+    pub fn args(&self) -> &Vec<String> {
+        &self.args
+    }
+
+    pub fn exe_filename(&self) -> &Path {
+        &self.exe_filename
+    }
+
+    pub fn working_directory(&self) -> Option<&Path> {
+        match &self.working_dir {
+            Some(path) => Some(&path),
+            None => None,
+        }
+    }
+    pub fn get_timeout(&self) -> Option<&Duration> {
+        match &self.timeout {
+            Some(timeout) => Some(&timeout),
+            None => None,
+        }
+    }
+
+    pub fn stderr(&self) -> &Vec<u8> {
+        &self.stderr
+    }
+    pub fn timed_out(&self) -> bool {
+        self.timed_out
+    }
+    pub fn exit_code(&self) -> Option<i32> {
+        self.exit_code
+    }
+    pub fn start(&self) -> &Option<DateTime<Utc>> {
+        &self.start
+    }
+    pub fn duration(&self) -> Option<Duration> {
+        self.duration
+    }
+
+    pub fn duration_string(&self) -> String {
+        let duration = match self.duration {
+            Some(duration) => {
+                let ms = duration.as_millis();
+                let mut elapsed = format!("{} ms", ms);
+                if ms > 999 {
+                    if ms < 90000 {
+                        elapsed = format!("{} s", ms / 1000);
+                    } else {
+                        elapsed = format!("{} m", ms / 60000);
+                    }
+                }
+                elapsed
+            }
+            None => format!(""),
+        };
+        duration
+    }
+    pub fn stdout(&self) -> &Vec<u8> {
+        &self.stdout
+    }
+
+    pub fn text(&self) -> Result<String> {
+        Ok(format!(
+            "{}\n{}",
+            std::str::from_utf8(&self.stdout)?,
+            std::str::from_utf8(&self.stderr)?
+        )
+        .trim()
+        .to_string())
+    }
+
+    pub fn status(&self) -> Status {
+        if self.timed_out {
+            Status::TimedOut
+        } else {
+            match self.exit_code() {
+                Some(exit_code) => {
+                    if exit_code == 0 {
+                        Status::Ok
+                    } else {
+                        Status::Error
+                    }
+                }
+                None => Status::Unknown,
+            }
+        }
+    }
+
+    pub fn user(&self) -> &str {
+        &self.user
+    }
+
+    pub fn machine(&self) -> &str {
+        &self.machine
+    }
+
+    pub fn os(&self) -> &str {
+        &self.os
+    }
+    pub fn matches(&self, search: &str) -> bool {
+        if search.len() == 0 {
+            true
+        } else {
+            let words = search.split_whitespace();
+            for word in words {
+                println!("word: {}", word);
+                if self.name.contains(word) {
+                    return true;
+                }
+                match self.text() {
+                    Ok(text) => {
+                        if text.contains(word) {
+                            return true;
+                        }
+                    }
+                    Err(_) => {}
+                }
+                for arg in &self.args {
+                    if arg.contains(word) {
+                        return true;
+                    }
+                }
+            }
+            false
+        }
+    }
+}
+
+// mutable methods
+impl Command {
     pub fn arg(&mut self, arg: &str) -> &mut Command {
         self.args.push(arg.to_string());
         self
@@ -133,6 +335,7 @@ impl Command {
                 os: whoami::distro(),
             };
             Storage::add(&command)?;
+            info!("{}", &command);
             Ok(command)
         } else {
             Err(anyhow!(
@@ -142,215 +345,42 @@ impl Command {
         }
     }
 
-    pub fn name(&self) -> &str {
-        &self.name
-    }
-
-    pub fn args(&self) -> &Vec<String> {
-        &self.args
-    }
-
-    pub fn exe_filename(&self) -> &Path {
-        &self.exe_filename
-    }
-
-    pub fn working_directory(&self) -> Option<&Path> {
-        match &self.working_dir {
-            Some(path) => Some(&path),
-            None => None,
-        }
-    }
-    pub fn get_timeout(&self) -> Option<&Duration> {
-        match &self.timeout {
-            Some(timeout) => Some(&timeout),
-            None => None,
-        }
-    }
-
-    pub fn timed_out(&self) -> bool {
-        self.timed_out
-    }
-    pub fn exit_code(&self) -> Option<i32> {
-        self.exit_code
-    }
-    pub fn start(&self) -> &Option<DateTime<Utc>> {
-        &self.start
-    }
-    pub fn duration(&self) -> Option<Duration> {
-        self.duration
-    }
-    pub fn stdout(&self) -> &Vec<u8> {
-        &self.stdout
-    }
-
     pub fn set_stdout(&mut self, stdout: Vec<u8>) {
         self.stdout = stdout
     }
-    pub fn stderr(&self) -> &Vec<u8> {
-        &self.stderr
-    }
 
     pub fn set_stderr(&mut self, stderr: Vec<u8>) {
         self.stderr = stderr
     }
-
-    pub fn text(&self) -> Result<String> {
-        Ok(format!(
-            "{}\n{}",
-            std::str::from_utf8(&self.stdout)?,
-            std::str::from_utf8(&self.stderr)?
-        )
-        .trim()
-        .to_string())
-    }
-
-    pub fn status(&self) -> Status {
-        if self.timed_out {
-            Status::TimedOut
-        } else {
-            match self.exit_code() {
-                Some(exit_code) => {
-                    if exit_code == 0 {
-                        Status::Ok
-                    } else {
-                        Status::Error
-                    }
-                }
-                None => Status::Unknown,
-            }
-        }
-    }
-
-    pub fn user(&self) -> &str {
-        &self.user
-    }
-
-    pub fn machine(&self) -> &str {
-        &self.machine
-    }
-
-    pub fn os(&self) -> &str {
-        &self.os
-    }
-    pub fn collect(search: &str, limit: usize) -> Result<Vec<Command>> {
-        Storage::search(search, limit)
-            .with_context(|| format!("Command::collect('{}',{})", search, limit))
-    }
-
-    pub fn matches(&self, search: &str) -> bool {
-        if search.len() == 0 {
-            true
-        } else {
-            let words = search.split_whitespace();
-            for word in words {
-                println!("word: {}", word);
-                if self.name.contains(word) {
-                    return true;
-                }
-                match self.text() {
-                    Ok(text) => {
-                        if text.contains(word) {
-                            return true;
-                        }
-                    }
-                    Err(_) => {}
-                }
-                for arg in &self.args {
-                    if arg.contains(word) {
-                        return true;
-                    }
-                }
-            }
-            false
-        }
-    }
-
-    fn which(name: &str) -> Result<PathBuf> {
-        let extensions = vec![".exe", ".bat", ".cmd", ""];
-        for ext in extensions.iter() {
-            let exe_name = format!("{}{}", name, ext);
-            match Command::which_exact(&exe_name) {
-                Some(path) => {
-                    return Ok(path);
-                }
-                None => {}
-            };
-        }
-        Err(anyhow!("unrecognized command '{}'", name))
-    }
-
-    fn which_exact(name: &str) -> Option<PathBuf> {
-        std::env::var_os("PATH").and_then(|paths| {
-            std::env::split_paths(&paths)
-                .filter_map(|dir| {
-                    let full_path = dir.join(&name);
-                    if full_path.is_file() {
-                        Some(full_path)
-                    } else {
-                        None
-                    }
-                })
-                .next()
-        })
-    }
-
-    pub fn format(&self, fmt: &str) -> String {
-        let duration = match self.duration {
-            Some(duration) => {
-                let ms = duration.as_millis();
-                let mut elapsed = format!("{} ms", ms);
-                if ms > 999 {
-                    if ms < 90000 {
-                        elapsed = format!("{} s", ms / 1000);
-                    } else {
-                        elapsed = format!("{} m", ms / 60000);
-                    }
-                }
-                elapsed
-            }
-            None => format!(""),
-        };
-        let cur_dir = match &self.working_dir {
-            Some(dir) => format!("{}", dir.display()),
-            None => "".to_string(),
-        };
-        if fmt == "%F" {
-            if cur_dir.len() > 0 {
-                format!(
+}
+impl Display for Command {
+    fn fmt(&self, f: &mut Formatter) -> std::fmt::Result {
+        let duration = self.duration_string();
+        match &self.working_dir {
+            Some(dir) => {
+                let cur_dir = format!("{}", dir.display());
+                write!(
+                    f,
                     "{} {} {} ({},{})",
                     &self.status(),
                     &self.name().yellow().bold(),
                     &self.args.join(" "),
                     cur_dir.white().bold(),
-                    duration.white().bold(),
-                )
-            } else {
-                format!(
-                    "{} {} {} ({})",
-                    &self.status(),
-                    &self.name().yellow().bold(),
-                    &self.args.join(" "),
-                    duration.white().bold(),
+                    duration.white().bold()
                 )
             }
-        } else {
-            format!("{}", self)
+            None => write!(
+                f,
+                "{} {} {} ({})",
+                &self.status(),
+                &self.name().yellow().bold(),
+                &self.args.join(" "),
+                duration.white().bold(),
+            ),
         }
     }
 }
 
-impl Display for Command {
-    fn fmt(&self, f: &mut Formatter) -> std::fmt::Result {
-        write!(
-            f,
-            "{} {} {}",
-            &self.status(),
-            &self.name(),
-            &self.args.join(" ")
-        )
-    }
-}
-
 impl Debug for Command {
     fn fmt(&self, f: &mut Formatter) -> std::fmt::Result {
         write!(f, "\n{:>15}: {}", "name", &self.name().green())?;
